/**
 * Displays images and shows information about those images
 * Created for the Jython Environment for Students (JES)
 * @author Keith McDermottt, gte047w@cc.gatech.edu
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;

public class PictureViewer implements MouseMotionListener, ActionListener, MouseListener
{
	//Main gui variables
	private JFrame pictureFrame;
	private JPanel panelLayout;
	private JScrollPane scrollPicture;
	
	//information bar variables
	private JLabel xValue;
	private JLabel yValue;
	private JLabel rValue;
	private JLabel gValue;
	private JLabel bValue;
	private JPanel colorLabel;

	//the menu components
	private JMenuBar menuBar;
	private JMenu zoomMenu;
	private JMenu updateMenu;
	private JMenuItem update;
	private JMenuItem twentyFive;
	private JMenuItem fifty;
	private JMenuItem seventyFive;
	private JMenuItem hundred;
	private JMenuItem hundredFifty;
	private JMenuItem twoHundred;
	private JMenuItem fiveHundred;
	
	//the picture currently and the orginal for zooming
	private JavaPicture picture;
	private JavaPicture orginalPicture;
    
    private ImageIcon scrollImageIcon;
    private JLabel imageLabel;
	
	//the zoom factor
	private double zoomFactor;
	
	//the number system to use
	//0 means starting at 0
	//1 means starting at 1
	private int numberBase=0;
	
	
	/**
     * Public constructor 
     * 
     * @param JavaPicture picture
     */
	public PictureViewer(JavaPicture picture)
	{
		this.picture=picture;
		this.orginalPicture=picture;
		
		zoomFactor=1;
		
		createWindow();
        
        imageLabel.addMouseMotionListener(this);
		imageLabel.addMouseListener(this);
	}

	/**
     * Changes the number system to start at one
     */
	public void changeToBaseOne()
	{
		numberBase=1;
	}
	/**
     * Set the title of the window
     *@param String title
     */
	public void setTitle(String title)
	{
		pictureFrame.setTitle(title);
	}
	
	/**
     * Creates the window
     */
	private void createWindow()
	{
		pictureFrame = new JFrame();//picture.getFileName());
		
		//create menu
		menuBar = new JMenuBar();
		zoomMenu = new JMenu("Zoom");
		updateMenu = new JMenu("Update Picture");
		update = new JMenuItem("Update");
		twentyFive = new JMenuItem("25%");
		fifty = new JMenuItem("50%");
		seventyFive = new JMenuItem("75%");
		hundred = new JMenuItem("100%");
		hundred.setEnabled(false);
		hundredFifty = new JMenuItem("150%");
		twoHundred = new JMenuItem("200%");
		fiveHundred = new JMenuItem("500%");
		
		update.addActionListener(this);
		twentyFive.addActionListener(this);
		fifty.addActionListener(this);
		seventyFive.addActionListener(this);
		hundred.addActionListener(this);
		hundredFifty.addActionListener(this);
		twoHundred.addActionListener(this);
		fiveHundred.addActionListener(this);
		
		updateMenu.add(update);
		zoomMenu.add(twentyFive);
		zoomMenu.add(fifty);
		zoomMenu.add(seventyFive);
		zoomMenu.add(hundred);
		zoomMenu.add(hundredFifty);
		zoomMenu.add(twoHundred);
		zoomMenu.add(fiveHundred);
		
		menuBar.add(zoomMenu);
		menuBar.add(updateMenu);
		
		
		
		pictureFrame.setJMenuBar(menuBar);
	    
	    //sets the frame size based on picture
		setFrameSize();
		pictureFrame.setResizable(true);
		pictureFrame.getContentPane().setLayout(new BorderLayout());
		pictureFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		//creates the information panel
		createInfoBar();
		
		//creates the scrollpane for the picture
		scrollPicture = new JScrollPane();
        scrollImageIcon = new ImageIcon(picture.bimg.getScaledInstance(picture.bimg.getWidth(), picture.bimg.getHeight(), Image.SCALE_FAST));
		imageLabel = new JLabel(scrollImageIcon);
        imageLabel.setVerticalAlignment(JLabel.TOP);
        imageLabel.setHorizontalAlignment(JLabel.LEFT);
        scrollPicture.setViewportView(imageLabel);
		pictureFrame.getContentPane().add(scrollPicture, BorderLayout.CENTER);
		
		pictureFrame.show();
	}
	
	/**
     * Create the frame size based on the picture
     *
     */
	private void setFrameSize()
	{
		int width;
		int height;
		
		if(picture.getWidth()<350)
		{
			width=350;
		}
		else if(picture.getWidth()>550)
		{
			width=550;
		}
		else
		{
			width=picture.getWidth()+12;
		}
		
		if(picture.getHeight()<225)
		{
			height=271;
		}
		else if(picture.getHeight()>550)
		{
			height=550;
		}
		else
		{
			height=picture.getHeight()+85;
		}
		pictureFrame.setSize(width,height);
	}	
	
	/**
     * Creates the North JPanel with all the information
     *
     */
	private void createInfoBar()
	{
		Font largerFont;
		panelLayout= new JPanel();
		panelLayout.setPreferredSize(new Dimension(350,30));
	
		xValue = new JLabel("X: N/A");
		yValue = new JLabel("Y: N/A");
		rValue = new JLabel("R: N/A");
		gValue = new JLabel("G: N/A");
		bValue = new JLabel("B: N/A");
		colorLabel = new JPanel();
		
		panelLayout.setBackground(Color.black);
		
		colorLabel.setBackground(Color.black);
		xValue.setForeground(Color.white);
		yValue.setForeground(Color.white);
		rValue.setForeground(Color.white);
		gValue.setForeground(Color.white);
		bValue.setForeground(Color.white);
		
		largerFont = new Font(xValue.getFont().getName(),xValue.getFont().getStyle(),14);
		xValue.setFont(largerFont);
		yValue.setFont(largerFont);
		rValue.setFont(largerFont);
		gValue.setFont(largerFont);
		bValue.setFont(largerFont);
		
		xValue.setPreferredSize(new Dimension(50,25));
		yValue.setPreferredSize(new Dimension(50,25));
		rValue.setPreferredSize(new Dimension(50,25));
		gValue.setPreferredSize(new Dimension(50,25));
		bValue.setPreferredSize(new Dimension(50,25));
		colorLabel.setPreferredSize(new Dimension(25,25));
		
		panelLayout.add(xValue);
		panelLayout.add(yValue);
		panelLayout.add(rValue);
		panelLayout.add(gValue);
		panelLayout.add(bValue);
		panelLayout.add(colorLabel);
		
		pictureFrame.getContentPane().add(panelLayout, BorderLayout.NORTH);
	}	
	
	/**
     * Zooms in the on picture by scaling the image.  
     * It is extremely memory intensive.
     *
     * @param double factor
     */
	public void zoom(double factor)
	{
		zoomFactor = factor;
		
		//Image tempImage = orginalPicture.getZoomImage();
		int width = (int) (orginalPicture.getWidth()*zoomFactor);
		int height = (int) (orginalPicture.getHeight()*zoomFactor);
		
		scrollImageIcon.setImage(picture.bimg.getScaledInstance(width, height, Image.SCALE_DEFAULT));
        //scrollPicture.setViewportView(picture);
		scrollPicture.revalidate();
		//picture.addMouseListener(this);
		//picture.addMouseMotionListener(this);
		pictureFrame.repaint();
		
	}
	
	/**
     * Repaints the image on the scrollpane.  Called from update picture
     */
	public void repaint()
	{
		zoom(zoomFactor);
		scrollPicture.repaint();
		scrollPicture.revalidate();
		pictureFrame.repaint();
	}
	
	private boolean checkImageValues(int x, int y)
	{
	
	
		if((int)(x/zoomFactor+numberBase)>= scrollImageIcon.getIconWidth()+numberBase ||
			(int)(y/zoomFactor+numberBase)>=scrollImageIcon.getIconHeight()+numberBase ||
			(int)(x/zoomFactor+numberBase)<numberBase || (int)(y/zoomFactor+numberBase)<numberBase)
		{
				return true;
		}
		
		return false;
	}
	
	//****************************************//
	//               Event Listeners          //
	//****************************************//
	
	/**
     * Controls the updating of the information values.
     *
     * @param MouseEvent e
     */
	public void mouseDragged(MouseEvent e)
	{
		try
		{
			//information for x y and rgb 
			//adjusts by zoomfactor to make pixels accurate
			//it reflects the pixels of the actual image
			//not the scaled image
			JavaPixel pixel = new JavaPixel(picture, (int)(e.getX()/zoomFactor+numberBase), (int)(e.getY()/zoomFactor+numberBase));
			if(this.checkImageValues(e.getX(), e.getY()))
			{
				xValue.setText("X: N/A");
				yValue.setText("Y: N/A");
				rValue.setText("R: N/A");
				gValue.setText("G: N/A");
				bValue.setText("B: N/A");
				colorLabel.setBackground(Color.black);
			}	
			else
			{
				xValue.setText("X: " + (int)(e.getX()/zoomFactor+numberBase));
				yValue.setText("Y: " + (int)(e.getY()/zoomFactor+numberBase));
				rValue.setText("R: " + pixel.getRed());
				gValue.setText("G: " + pixel.getGreen());
				bValue.setText("B: " + pixel.getBlue());
				colorLabel.setBackground(new Color(pixel.getRed(), pixel.getGreen(), pixel.getBlue()));
			}
			
			
		}
		catch(ArrayIndexOutOfBoundsException exception)
		{
			xValue.setText("X: N/A");
			yValue.setText("Y: N/A");
			rValue.setText("R: N/A");
			gValue.setText("G: N/A");
			bValue.setText("B: N/A");
			colorLabel.setBackground(Color.black);

		}
		catch(NullPointerException exception)
		{
		}
		
		
	}
	
	public void mouseMoved(MouseEvent e)
	{
	}
	
	public void mouseClicked(MouseEvent e)
	{
	}
	/**
     * Controls the updating of the information values.
     *
     * @param MouseEvent e
     */	
	public void mousePressed(MouseEvent e)
	{
		try
		{
			//information for x y and rgb 
			//adjusts by zoomfactor to make pixels accurate
			//it reflects the pixels of the actual image
			//not the scaled image
			JavaPixel pixel = new JavaPixel(picture, e.getX(), e.getY());
			if(this.checkImageValues(e.getX(), e.getY()))
			{
				xValue.setText("X: N/A");
				yValue.setText("Y: N/A");
				rValue.setText("R: N/A");
				gValue.setText("G: N/A");
				bValue.setText("B: N/A");
				colorLabel.setBackground(Color.black);
			}	
			else
			{
				xValue.setText("X: " + (int)(e.getX()/zoomFactor+numberBase));
				yValue.setText("Y: " + (int)(e.getY()/zoomFactor+numberBase));
				rValue.setText("R: " + pixel.getRed());
				gValue.setText("G: " + pixel.getGreen());
				bValue.setText("B: " + pixel.getBlue());
				colorLabel.setBackground(new Color(pixel.getRed(), pixel.getGreen(), pixel.getBlue()));
			}
			
			
		}
		catch(ArrayIndexOutOfBoundsException exception)
		{
			xValue.setText("X: N/A");
			yValue.setText("Y: N/A");
			rValue.setText("R: N/A");
			gValue.setText("G: N/A");
			bValue.setText("B: N/A");
			colorLabel.setBackground(Color.black);

		}
		catch(NullPointerException exception)
		{
		}
	}
	
	public void mouseReleased(MouseEvent e)
	{
	}
	
	public void mouseEntered(MouseEvent e)
	{
	}
	
	public void mouseExited(MouseEvent e)
	{
	}
	/**
     * Controls the zoom menu bar
     *
     * @param ActionEvent a
     */
	public void actionPerformed(ActionEvent a)
	{
		
		if(a.getActionCommand().equals("Update"))
		{
			this.repaint();
		}
		
		if(a.getActionCommand().equals("25%"))
		{
			this.zoom(.25);
			twentyFive.setEnabled(false);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("50%"))
		{
			this.zoom(.50);
			twentyFive.setEnabled(true);
			fifty.setEnabled(false);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("75%"))
		{
			this.zoom(.75);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(false);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("100%"))
		{
			this.zoom(1.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(false);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("150%"))
		{
			this.zoom(1.5);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(false);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("200%"))
		{
			this.zoom(2.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(false);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("500%"))
		{
			this.zoom(5.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(false);
		}
	}
	
	//****************************************//
	//              Test Main                 //
	//****************************************//
	
	/**
     * Test Main
     */
	public static void main( String args[])
    {
		JavaPicture p = new JavaPicture();
		p.loadImage("c:\\School Work\\Fall_Research\\kitty.jpg");
		
		PictureViewer test;
		test= new PictureViewer(p);
		test.changeToBaseOne();
		
		for(int x=0;x<p.getWidth();x++)
		{
			for(int y=0;y<p.getHeight();y++)
			{
				p.getPixel(x,y).setRed(255);
			}
		}
		
		System.out.println(p.getWidth());
		
		//test.zoom(2);
    }
}