#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information
#5/16/03: updated save() and saveAs() to return success booleans. -AdamW
#5/20/03: added loadSuccess to be called when a JESThread has successfully loaded
#         code. - AdamW



import JESAbout
import JESIntroduction
import JESExceptionRecord
import JESRunnable
import JESConstants
import JESInterpreter
import JESStdOutputBuffer
import JESUI
import os
import java.io as io
import java.lang as lang
import javax.swing as swing
import string
import sys
import JESTabnanny
import JavaMusic
from code import compile_command
from tokenize import TokenError



class JESProgram:
################################################################################
# Function name: __init__
# Return:
# Description:
#
################################################################################
    def __init__(self):
        #"@sig public JESProgram()"
       #swing.UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
    
        self.userExperience =JESConstants.BEGINNER_MODE
        self.gutterOn = 1
        self.blockBoxOn = 1
        self.userFont=self.grabFontSize()
        self.preCheckForConfigFile()
        self.textForCommandWindow = ''
        self.aboutWindow = None
        self.introWindow = None
        self.gui = JESUI.JESUI(self)
        self.filename = ' '
        self.settingsFileName=''
        self.interpreter = JESInterpreter.JESInterpreter(self)
	
        self.varsToHighlight = self.interpreter.getVarsToHighlight()

        self.chooser = swing.JFileChooser()
        self.defaultPath = self.chooser.getCurrentDirectory()
        self.setHelpArray()
        self.loadSuccess()
        self.gui.show()
        if self.blockBoxOn:
            self.gui.editor.addBox()
        elif self.gutterOn == 0:
            self.gui.editor.removeBox()
        if self.gutterOn:
            self.gui.turnOnGutter()
        elif self.gutterOn == 0:
            self.gui.turnOffGutter()
	self.checkForConfigFile()
            
        #self.gui.repaint()
        
       
	#JavaMusic.open()
    def getVarsToHighlight(self):
        return self.varsToHighlight

        
#main
    def main(self, args):
        "@sig public static void main(String args[])"
        self.__init__()
        return self

################################################################################
# Function name: newFile
# Description: 
#     Blanks outthe fileName and the text in the editor window. 
################################################################################
    def newFile(self):
        self.gui.editor.setText('')
        self.gui.setFileName('')
        self.chooser = swing.JFileChooser() #reset the filechooser to keep

################################################################################
# Function name: openFile
# Description: 
#     This function will open a dialog window allowing the user to select a 
#     file.  If the user selects a file, it is opened, and its contents are  
#     place into the text editor. 
################################################################################
    def openFile(self):
        self.chooser = swing.JFileChooser(self.defaultPath)
        self.chooser.setApproveButtonText("Open File")
        returnVal = self.chooser.showOpenDialog(self.gui)
        if returnVal ==  0: #User has chosen a file, so now it can be opened
            file = open(self.chooser.getSelectedFile().getPath(),'r')
            self.filename = file.name
            self.gui.setFileName(self.chooser.getSelectedFile().getName())
            self.gui.editor.setText(file.read())
            self.gui.editor.modified = 0
	    self.gui.loadDifferent()
            file.close()
            self.defaultPath = self.chooser.getCurrentDirectory()
            
      
################################################################################
# Function name: saveFile
# Description:  
#     This function is called when the user selects the save file option from 
#     the user menu.  If there is no file opened yet, the function will do          
#     nothing.  If the file has no name, the function will call saveAs.  Else, 
#     it will write the contents of the text editor to the file of the current 
#     name.
################################################################################
    def saveFile(self):
        try:
            if self.filename != '' :
                text=self.gui.editor.getText()
                #self.chooser.setCurrentDirectory(self.defaultPath)
                #file = open(self.chooser.getSelectedFile().getPath(),'w+')
                #David - testing something out
                #text = text.splitlines(1)
                #file.writelines(text)
                #self.filename = file.name
                #file.close()
                #Commented out by AW: Trying to see if using java instead of jython
                #gets rid of the newline errors
                
                filePath = self.chooser.getSelectedFile().getPath()
                self.filename =  os.path.normpath(filePath)
                fileWriter = io.FileWriter(filePath, 0)
                fileWriter.write(text)
                fileWriter.close()
                
                self.defaultPath = self.chooser.getCurrentDirectory()
                self.gui.editor.modified = 0
                self.gui.setFileName(os.path.basename(self.filename))
		return 1
            else:
                return self.saveAs()
        except:
            #Error handling for saveFile
            return self.saveAs()

################################################################################
# Function name: saveAs
# Description: 
#     Opens a dialog and allows users to select or either type in a filename to
#     save the file as.  If the user selects cancel, nothing happens.
################################################################################
    def saveAs(self):
       try:
        self.chooser.setCurrentDirectory(self.defaultPath)
        text = self.gui.editor.getText()
        self.chooser.setApproveButtonText("Save File")
        returnVal = self.chooser.showSaveDialog(self.gui)
        if returnVal ==  0: #User has chosen a file, so now it can be saved
            #DNR
            #file = open(self.chooser.getSelectedFile().getPath(),'w+')
            #self.gui.setFileName(self.chooser.getSelectedFile().getName())
            #text = text.splitlines(1)
            #file.writelines(text)
            #self.filename = file.name
            #file.close()
            #Commented out by AW: Trying to see if using java instead of jython
            #gets rid of the newline errors
            
            filePath = self.chooser.getSelectedFile().getPath()
            self.filename = os.path.normpath(filePath)
            fileWriter = io.FileWriter(filePath, 0)
            fileWriter.write(text)
            fileWriter.close()
            
            self.defaultPath = self.chooser.getCurrentDirectory()
            self.gui.editor.modified = 0
            self.gui.setFileName(os.path.basename(self.filename))
	    return 1
       except:
       #TODO - fix
         #Error handling for saveAs
         print "Error coming from saveAs"
	 return 0

################################################################################
# Function name: checkForSave
# Description:
#
################################################################################
    def checkForSave(self):
        pass

################################################################################
# Function name: loadFile
# Description: checks for possible errors, then calls JESInterpreter's load file
#              function
#
#              the errors are:
#              1) the program.filename field can contain no file name (ie, a
#                 null string)
#              2) program.filename is a valid string, but the file no longer
#                 exists
#              3) JESTabnanny throws a Token Error - some kind of problem parsing
#                 the file.  has something to do with unbalanced parenthesis
#              4) The file contains ambigious indentation
#        If any of these errors are present, a JESExceptionRecord is created,
#        and a JESRunnable is called and the code is not loaded by the interpreter
#       Otherwise, the code is loaded.
################################################################################
    def loadFile(self):
        if self.filename == ' ':
            self.setErrorByHand(JESConstants.JESPROGRAM_NO_FILE, 0)
        else: # error 1. didn't occur
          
            try:
                file = open(self.filename, 'r')
                fileText = file.read()
                file.close()
            except:
                self.setErrorByHand(JESConstants.JESPROGRAM_ERROR_LOADING_FILE +   self.filename + '\n',0)

            else: # error 2. didn't occur
                self.gui.commandWindow.setKeymap(None)
                try:
                    lineWithError = JESTabnanny.check(self.filename)

      
                except:

                    import sys
                    a,b,c = sys.exc_info()

                    self.setErrorFromUserCode( a,b,c )
                    return

                #this is clugy
                #JESTabnanny can either throw an exception,
                # or return a line number.
                # both signal an error, and we handle them seperatly
                # error 3. didn't occur
                if not lineWithError is None:
                    # an error has occured in the file
                    self.setErrorByHand(JESConstants.TAB_ERROR_MESSAGE +'%d\n' % lineWithError,
                                         lineWithError)

                    return

                # error 4. didn't occur,
		# give the command area the focus
                self.interpreter.load( self.filename)	
		self.gui.commandWindow.requestFocus()
                self.gui.editor.getDocument().removeErrorHighlighting()

    def setErrorByHand(self,message,lineNumber):
        excRecord = JESExceptionRecord.JESExceptionRecord(self.filename,self)
	if message == JESConstants.JESPROGRAM_NO_FILE:
		excRecord.setByHand(message)
	else:
		excRecord.setByHand(JESConstants.TAB_ERROR_MESSAGE + \
                 	           '%d\n' % lineNumber,
                        	    lineNumber
                           	   )	
        runnable = JESRunnable.JESRunnable(self.interpreter,'',excRecord,'run')
        runnable.run()

    def setErrorFromUserCode(self,type,value,trace):
        excRecord = JESExceptionRecord.JESExceptionRecord(self.filename,self)
        excRecord.setFromUserCode(type,value,trace)
        runnable = JESRunnable.JESRunnable(self.interpreter,
                                           '',
                                           excRecord,
                                           'run')
        runnable.run()

        
        
################################################################################
# Function name: debugger_paused
# Parameters:
#     None
# Description: 
#     sets the Interpreter and the JES gui into debugger mode, the debugger
#     made a stop
################################################################################
    def goto_debugger():
        debugger_mode = true

################################################################################
# Function name: editorLoaded
# Description:
#     returns whether the text editor has been loaded
#     
################################################################################

    def editorLoaded(self):
        return self.gui.loadButton.getForeground() == JESConstants.LOAD_BUTTON_DIFF_COLOR

################################################################################
# Function name: loadSuccess
# Description:
#     Called whenever the interpreter successfully loads a Editor Document.
#     5/20/03 AdamW: Used for recoloring the Load Button on load success
################################################################################
    def loadSuccess(self):
        #self.gui.refreshDebugMenu()
	self.gui.loadCurrent()
        

################################################################################
# Function name: checkTabs
# Description:
#
################################################################################
    def checkTabs(self):

        if self.filename != '':
            #tabnanny sends the response to stdout, this redirects that output

            badLine = JESTabnanny.check(self.filename)
            # tabnanny calls tokenizer.tokenize, which can
            # throw a TokenError exception
            # we catch it when we call checkTabs

            return badLine
        return ''
################################################################################
# Function name: closeProgram
# Description:
#     Exits JES
################################################################################
    def closeProgram(self):
        lang.System.exit(0)

################################################################################
# Function name: stopThread
# Description:
#
################################################################################
    def stopThread(self):
        self.interpreter.stopThread()

################################################################################
# Function name: openAboutWindow
# Description: 
#     Opens up a JESAboutWindow.
################################################################################
    def openAboutWindow(self):
        if self.aboutWindow == None:
            aboutWindow = JESAbout.JESAbout()
        aboutWindow.show()

################################################################################
# Function name: openIntroductionWindow
# Description: 
#     Opens up a JESIntroductionWindow.
################################################################################
    def openIntroductionWindow(self):
        if self.introWindow == None:
            introWindow = JESIntroduction.JESIntroduction()
        introWindow.show()
   
################################################################################
# Function name: openSettingsGUI
# Description: 
#     Opens up a settingd GUI Dialog.
################################################################################
    def openSettingsGUI(self):
        self.gui.openSettings()
################################################################################
# Function Name: setHelpArray
# Description: 
#     This sets the array of help file names in self.gui.  If the folder 
#     JESHelp cannot be located, the function will throw an exception which is 
#     caught by the function.
################################################################################
    def setHelpArray(self):
        try:
            myFile= io.File(JESConstants.JESHELP)
            arrayOfFiles = myFile.listFiles()
            arrayOfNames = []
            for x in arrayOfFiles:
                arrayOfNames.append(x.toURL())
            stringNames = []
            for x in arrayOfNames:
                stringNames.append(x.toString())
            stringNames.sort()
            for name in stringNames:
                if (name.find('.DS_Store') != -1):
                    stringNames.remove(name)
            l = stringNames[1]
            stringNames.remove(l)
            stringNames.insert(10, l)
            self.gui.SetHelpFiles(stringNames)
        except:
            print "ERROR opening help files"

################################################################################
# Function name: runCommand
# Parameters:
#     -text:
# Description:
#
################################################################################
    def runCommand(self, text):
	#If the document in the editor is not current with the interpreter, warn
	#the user.
        if self.editorLoaded():
	    self.gui.commandWindow.showText(JESConstants.EDITOR_LOAD_WARNING)

        self.gui.commandWindow.setKeymap(None)
        self.interpreter.runCommand(text)
 

################################################################################
# Function name: sendTextToCommandWindow
# Parameters: 
#     -text:
# Description:
#     As the interpreter runs, it will generate text for the command window to 
#     display.  That text is buffered in JESProgram until the command window 
#     asks for it with the getTextForCommandWindow function
################################################################################
    def sendTextToCommandWindow(self, text):
        self.textForCommandWindow = self.textForCommandWindow + text

################################################################################
# Function name: getTextForCommandWindow
# Description: 
#     Returns the buffered text that was generated by JESInterpreter
################################################################################
    def getTextForCommandWindow(self):
        returnText = self.textForCommandWindow
        self.textForCommandWindow = ''
        return returnText

################################################################################
# Function name: sendErrorToCommandWindow
# Parameters:
#     -text:
# Description:
#
################################################################################
    def sendErrorToCommandWindow(self, text):
        self.textForCommandWindow = self.textForCommandWindow + text

################################################################################
# Function name: sendError
# Parameters: self
# Description: sends an error tp the command window
#
################################################################################
    def sendError(self):
        errFp = StringIO.StringIO()
        traceback.print_exc( file=errFp )
        errMsg = errFp.getvalue()
        self.sendErrorToCommandWindow( errMsg)

################################################################################
# Function name:  checkForConfigFile
# Parameters: self
# Description: Checks to see if the config file already exists.  If it does exist,
#      the function closes the file and exits so that JES can run normally.  If the 
#      file does not exist, an introduction to JES is brought up explaining to the 
#      user about the settings file for JES.  The settings menu option is also 
#      brought up so that the user can create the settings file.
#
################################################################################
    def checkForConfigFile(self):
        try:
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME)
            array= self.readFromConfigFile()
            self.userExperience=array[JESConstants.CONFIG_MODE]
            self.userFont=int(array[JESConstants.CONFIG_FONT])
            self.gutterOn = int(array[JESConstants.CONFIG_GUTTER])
            self.blockBoxOn = int(array[JESConstants.CONFIG_BLOCK])
            self.webDefinitions = array[JESConstants.CONFIG_WEB_TURNIN]
            f.close()
        except:
            #Config FIle doesn't exist, need to create, also need to introduce
            #the user to JES.
            self.gutterOn = 1
            self.blockBoxOn = 1
            self.webDefinitions = JESConstants.WEB_DEFINITIONS
            self.writeToConfigFile('','','',JESConstants.BEGINNER_MODE, JESConstants.MID_FONT, '')
            self.openSettingsGUI()
            self.openIntroductionWindow()
            
    def preCheckForConfigFile(self):
        try:
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME)
            array= self.readFromConfigFile()
            self.userExperience=array[JESConstants.CONFIG_MODE]
            self.userFont=int(array[JESConstants.CONFIG_FONT])
            self.gutterOn = int(array[JESConstants.CONFIG_GUTTER])
            self.blockBoxOn = int(array[JESConstants.CONFIG_BLOCK])
            self.webDefinitions = int(array[JESCosntants.CONFIG_WEB_TURNIN])
            f.close()
        except:
            pass

################################################################################
# Function name: writeToConfigFile
# Parameters: self, name, gt, mail
# Description: Writes the passed in information to the config file.  If the file 
#        exists, it will be over written.  If it does not exist, it will be created.
#        If an IO Error occurs, an error message is printed to the transcript.
#
################################################################################
    def writeToConfigFile(self,name,gt,mail,usermode,font,email):
        if mail == None:
            mail = ''
        if email == None:
            email = ''
        try:
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME,'w')
            f.write(name)
            f.write("\n")
            f.write(gt)
            f.write("\n")
            f.write(mail)
            f.write("\n")
            f.write(usermode)
            f.write("\n")
            try:
                font=font.toString()
            except:
                pass
            f.write(font)
            f.write("\n")
            f.write(email)
            f.write("\n")
            f.write(str(self.gutterOn))
            f.write("\n")
            f.write(str(self.blockBoxOn))
            f.write("\n")
            f.write(self.webDefinitions)
            f.write("\n")
            f.close()
        except:
            print 'Warning.  An unexpected IO Error has occurred in writeToConfigFile in JESProgram' 

################################################################################
# Function name: writeConfigListToFile
# Parameters:
#     -array: an array of preperties to be written to the file
# Description:
#     Takes an array of an arbitrary amount of properties and writes them out to
#     the config file, filling in any missing properties from the old version of
#     the configuration file.  The property list is in the format:
#     [[JESConstants.CONFIG_GT, gtXXXXX], [JESConstants.CONFIG_GUTTER, 0], ...]
################################################################################
    def writeConfigListToFile(self, array):
        try:
            oldarray = self.readFromConfigFile()
            for i in range(0, len(oldarray)):
                for element in array:
                    if element[0] == i:
                        oldarray[i] = element[1]
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME,'w')
            #print oldarray
            for element in oldarray:
                f.write(str(element))
                f.write("\n")
            f.close()
        except Exception, e:
            print 'Warning: An unexpecter IO Error has occurred in writeConfigListToFile in JESProgram' 



################################################################################
# Function name: readFromConfigFile
# Parameters: self
# Description: Attempts to open the Configfile.  If it exists, it is opened and
#       read into an array.  Each line of the file will get its spot in the array
#       and newline characters will be removed.  The array is returned.  The 
#       configfile should exist before this function is called.  If an IO Error 
#       occurs, a message will be printed to the transcript. 
#
################################################################################
    def readFromConfigFile(self):
        try:
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME,'r')
            text=f.read()
            f.close()
            array=text.splitlines()
            return array
        except:
            pass

    def grabSubmissionAddress():
        return JESConstants.TO_ADDR

    def grabFontSize(self):
        try:
            homedir=os.path.expanduser("~")
            f=open(homedir+io.File.separator+JESConstants.JES_CONFIG_FILE_NAME,'r')
            text=f.read()
            f.close()
            array=text.splitlines()
            return string.atoi(array[4])
        except:
            return JESConstants.FONT_SIZE       
            
if __name__ == '__main__':   
    mainJESProgram = JESProgram()